@@ -11,26 +11,42 @@ import re
 import shutil
 import tempfile
 import typing
+import warnings
 import zipfile
 from pathlib import Path
 from typing import Optional, Sequence, Tuple, Union, cast
 from urllib.parse import urlparse
 
+import boto3
 import requests
+import torch
+from botocore import UNSIGNED
+from botocore.config import Config
 from tqdm import tqdm as _tqdm
 
 import flair
 
 logger = logging.getLogger("flair")
 
+url_proxies: Optional[typing.Dict[str, str]] = None
 
-def load_big_file(f: str) -> mmap.mmap:
+
+def set_proxies(proxies: typing.Dict[str, str]) -> None:
     """
-    Workaround for loading a big pickle file. Files over 2GB cause pickle errors on certin Mac and Windows distributions.
+    Allows for data downloaded from urls to be forwarded to a proxy, see https://requests.readthedocs.io/en/latest/user/advanced/#proxies
+    :param proxies: A dictionary of proxies according to the requests documentation.
+    :return: None
+    """
+    global url_proxies
+    url_proxies = proxies
+
+
+def load_big_file(f: str):
+    """
+    Workaround for loading a big pickle file. Files over 2GB cause pickle errors on certain Mac and Windows distributions.
     :param f:
     :return:
     """
-    logger.info(f"loading file {f}")
     with open(f, "rb") as f_in:
         # mmap seems to be much more memory efficient
         bf = mmap.mmap(f_in.fileno(), 0, access=mmap.ACCESS_READ)
@@ -82,19 +98,24 @@ def cached_path(url_or_filename: str, cache_dir: Union[str, Path]) -> Path:
     return the path to the cached file. If it's already a local path,
     make sure the file exists and then return the path.
     """
-    if type(cache_dir) is str:
-        cache_dir = Path(cache_dir)
-    dataset_cache = flair.cache_root / cache_dir
+    cache_dir = Path(cache_dir)
+
+    if flair.cache_root not in cache_dir.parents:
+        dataset_cache = flair.cache_root / cache_dir
+    else:
+        dataset_cache = cache_dir
 
     parsed = urlparse(url_or_filename)
 
     if parsed.scheme in ("http", "https"):
         # URL, so get it from the cache (downloading if necessary)
         return get_from_cache(url_or_filename, dataset_cache)
-    elif parsed.scheme == "" and Path(url_or_filename).exists():
+    elif parsed.scheme == "s3":
+        return download_s3_to_path(parsed.netloc, dataset_cache)
+    elif len(parsed.scheme) < 2 and Path(url_or_filename).exists():
         # File, and it exists.
         return Path(url_or_filename)
-    elif parsed.scheme == "":
+    elif len(parsed.scheme) < 2:
         # File, but it doesn't exist.
         raise FileNotFoundError("file {} not found".format(url_or_filename))
     else:
@@ -102,6 +123,21 @@ def cached_path(url_or_filename: str, cache_dir: Union[str, Path]) -> Path:
         raise ValueError("unable to parse {} as a URL or as a local path".format(url_or_filename))
 
 
+def download_s3_to_path(bucket_name: str, cache_path: Path) -> Path:
+    out_path = cache_path / bucket_name
+    if out_path.exists():
+        return out_path
+    s3 = boto3.resource("s3", config=Config(signature_version=UNSIGNED))
+    bucket = s3.Bucket(bucket_name)
+    for obj in bucket.objects.iterator():
+        if obj.key[-1] == "/":
+            continue
+        target = out_path / obj.key
+        target.parent.mkdir(exist_ok=True, parents=True)
+        bucket.download_file(obj.key, str(target))
+    return out_path
+
+
 def unzip_file(file: Union[str, Path], unzip_to: Union[str, Path]):
     from zipfile import ZipFile
 
@@ -160,41 +196,6 @@ def unpack_file(file: Path, unpack_to: Path, mode: str = None, keep: bool = True
         os.remove(str(file))
 
 
-def download_file(url: str, cache_dir: Union[str, Path]):
-    cache_dir = Path(cache_dir)
-    cache_dir.mkdir(parents=True, exist_ok=True)
-
-    filename = re.sub(r".+/", "", url)
-    # get cache path to put the file
-    cache_path = cache_dir / filename
-
-    # Download to temporary file, then copy to cache dir once finished.
-    # Otherwise you get corrupt cache entries if the download gets interrupted.
-    fd, temp_filename = tempfile.mkstemp()
-    logger.info("%s not found in cache, downloading to %s", url, temp_filename)
-
-    # GET file object
-    req = requests.get(url, stream=True)
-    content_length = req.headers.get("Content-Length")
-    total = int(content_length) if content_length is not None else None
-    progress = Tqdm.tqdm(unit="B", total=total)
-    with open(temp_filename, "wb") as temp_file:
-        for chunk in req.iter_content(chunk_size=1024):
-            if chunk:  # filter out keep-alive new chunks
-                progress.update(len(chunk))
-                temp_file.write(chunk)
-
-    progress.close()
-
-    logger.info("copying %s to cache at %s", temp_filename, cache_path)
-    shutil.copyfile(temp_filename, str(cache_path))
-    logger.info("removing temp file %s", temp_filename)
-    os.close(fd)
-    os.remove(temp_filename)
-
-    progress.close()
-
-
 # TODO(joelgrus): do we want to do checksums or anything like that?
 def get_from_cache(url: str, cache_dir: Path) -> Path:
     """
@@ -224,10 +225,10 @@ def get_from_cache(url: str, cache_dir: Path) -> Path:
         logger.info("%s not found in cache, downloading to %s", url, temp_filename)
 
         # GET file object
-        req = requests.get(url, stream=True, headers={"User-Agent": "Flair"})
+        req = requests.get(url, stream=True, headers={"User-Agent": "Flair"}, proxies=url_proxies)
         content_length = req.headers.get("Content-Length")
         total = int(content_length) if content_length is not None else None
-        progress = Tqdm.tqdm(unit="B", total=total)
+        progress = Tqdm.tqdm(unit="B", total=total, unit_scale=True, unit_divisor=1024)
         with open(temp_filename, "wb") as temp_file:
             for chunk in req.iter_content(chunk_size=1024):
                 if chunk:  # filter out keep-alive new chunks
@@ -252,15 +253,37 @@ def open_inside_zip(
     encoding: str = "utf8",
 ) -> typing.Iterable:
     cached_archive_path = cached_path(archive_path, cache_dir=Path(cache_dir))
-    archive = zipfile.ZipFile(cached_archive_path, "r")
-    if member_path is None:
-        members_list = archive.namelist()
-        member_path = get_the_only_file_in_the_archive(members_list, archive_path)
-    member_path = cast(str, member_path)
-    member_file = archive.open(member_path, "r")
+    with zipfile.ZipFile(cached_archive_path, "r") as archive:
+        if member_path is None:
+            members_list = archive.namelist()
+            member_path = get_the_only_file_in_the_archive(members_list, archive_path)
+        member_path = cast(str, member_path)
+        member_file = archive.open(member_path, "r")
     return io.TextIOWrapper(member_file, encoding=encoding)
 
 
+def extract_single_zip_file(
+    archive_path: str,
+    cache_dir: Union[str, Path],
+    member_path: Optional[str] = None,
+) -> Path:
+    cache_dir = Path(cache_dir)
+    cached_archive_path = cached_path(archive_path, cache_dir=cache_dir)
+    if flair.cache_root not in cache_dir.parents:
+        dataset_cache = flair.cache_root / cache_dir
+    else:
+        dataset_cache = cache_dir
+    with zipfile.ZipFile(cached_archive_path, "r") as archive:
+        if member_path is None:
+            members_list = archive.namelist()
+            member_path = get_the_only_file_in_the_archive(members_list, archive_path)
+        output_path = dataset_cache / member_path
+
+        if not output_path.exists():
+            archive.extract(member_path, dataset_cache)
+        return output_path
+
+
 def get_the_only_file_in_the_archive(members_list: Sequence[str], archive_path: str) -> str:
     if len(members_list) > 1:
         raise ValueError(
@@ -320,3 +343,13 @@ def instance_lru_cache(*cache_args, **cache_kwargs):
         return create_cache
 
     return decorator
+
+
+def load_torch_state(model_file: str) -> typing.Dict[str, typing.Any]:
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore")
+        # load_big_file is a workaround byhttps://github.com/highway11git
+        # to load models on some Mac/Windows setups
+        # see https://github.com/zalandoresearch/flair/issues/351
+        f = load_big_file(model_file)
+        return torch.load(f, map_location="cpu")
