@@ -1,18 +1,18 @@
 import logging
 from pathlib import Path
-from typing import List, Tuple, Union
+from typing import Any, Dict, List, Union
 
 import torch
 
 import flair.embeddings
 import flair.nn
-from flair.data import DataPoint, Sentence
+from flair.data import Sentence
 from flair.file_utils import cached_path
 
 log = logging.getLogger("flair")
 
 
-class TextClassifier(flair.nn.DefaultClassifier[Sentence]):
+class TextClassifier(flair.nn.DefaultClassifier[Sentence, Sentence]):
     """
     Text Classification Model
     The model takes word embeddings, puts them into an RNN to obtain a text
@@ -23,13 +23,13 @@ class TextClassifier(flair.nn.DefaultClassifier[Sentence]):
 
     def __init__(
         self,
-        document_embeddings: flair.embeddings.DocumentEmbeddings,
+        embeddings: flair.embeddings.DocumentEmbeddings,
         label_type: str,
         **classifierargs,
     ):
         """
         Initializes a TextClassifier
-        :param document_embeddings: embeddings used to embed each data point
+        :param embeddings: embeddings used to embed each data point
         :param label_dictionary: dictionary of labels you want to predict
         :param multi_label: auto-detected by default, but you can set this to True to force multi-label prediction
         or False to force single-label prediction
@@ -40,47 +40,27 @@ class TextClassifier(flair.nn.DefaultClassifier[Sentence]):
         """
 
         super(TextClassifier, self).__init__(
-            **classifierargs, final_embedding_size=document_embeddings.embedding_length
+            **classifierargs,
+            embeddings=embeddings,
+            final_embedding_size=embeddings.embedding_length,
         )
 
-        self.document_embeddings: flair.embeddings.DocumentEmbeddings = document_embeddings
-
         self._label_type = label_type
 
         # auto-spawn on GPU if available
         self.to(flair.device)
 
-    def forward_pass(
-        self,
-        sentences: Union[List[Sentence], Sentence],
-        for_prediction: bool = False,
-    ) -> Union[Tuple[torch.Tensor, List[List[str]]], Tuple[torch.Tensor, List[List[str]], List[DataPoint]]]:
-        if not isinstance(sentences, list):
-            sentences = [sentences]
+    def _get_embedding_for_data_point(self, prediction_data_point: Sentence) -> torch.Tensor:
+        embedding_names = self.embeddings.get_names()
+        return prediction_data_point.get_embedding(embedding_names)
 
-        # embed sentences
-        self.document_embeddings.embed(sentences)
-
-        # make tensor for all embedded sentences in batch
-        embedding_names = self.document_embeddings.get_names()
-        text_embedding_list = [sentence.get_embedding(embedding_names).unsqueeze(0) for sentence in sentences]
-        text_embedding_tensor = torch.cat(text_embedding_list, 0).to(flair.device)
-
-        labels: List[List[str]] = []
-        for sentence in sentences:
-            labels.append([label.value for label in sentence.get_labels(self.label_type)])
-
-        if for_prediction:
-            sentences_for_prediction: List[DataPoint] = []
-            sentences_for_prediction.extend(sentences)
-            return text_embedding_tensor, labels, sentences_for_prediction
-
-        return text_embedding_tensor, labels
+    def _get_data_points_from_sentence(self, sentence: Sentence) -> List[Sentence]:
+        return [sentence]
 
     def _get_state_dict(self):
         model_state = {
             **super()._get_state_dict(),
-            "document_embeddings": self.document_embeddings,
+            "document_embeddings": self.embeddings.save_embeddings(use_state_dict=False),
             "label_dictionary": self.label_dictionary,
             "label_type": self.label_type,
             "multi_label": self.multi_label,
@@ -91,26 +71,26 @@ class TextClassifier(flair.nn.DefaultClassifier[Sentence]):
 
     @classmethod
     def _init_model_with_state_dict(cls, state, **kwargs):
+        import re
 
-        weights = None if "weight_dict" not in state.keys() else state["weight_dict"]
-        label_type = None if "label_type" not in state.keys() else state["label_type"]
+        # remap state dict for models serialized with Flair <= 0.11.3
+        state_dict = state["state_dict"]
+        for key in list(state_dict.keys()):
+            state_dict[re.sub("^document_embeddings\\.", "embeddings.", key)] = state_dict.pop(key)
 
         return super()._init_model_with_state_dict(
             state,
-            document_embeddings=state["document_embeddings"],
-            label_dictionary=state["label_dictionary"],
-            label_type=label_type,
-            multi_label=state["multi_label"],
-            multi_label_threshold=0.5
-            if "multi_label_threshold" not in state.keys()
-            else state["multi_label_threshold"],
-            loss_weights=weights,
+            embeddings=state.get("document_embeddings"),
+            label_dictionary=state.get("label_dictionary"),
+            label_type=state.get("label_type"),
+            multi_label=state.get("multi_label"),
+            multi_label_threshold=state.get("multi_label_threshold", 0.5),
+            loss_weights=state.get("weight_dict"),
             **kwargs,
         )
 
     @staticmethod
     def _fetch_model(model_name) -> str:
-
         model_map = {}
         hu_path: str = "https://nlp.informatik.hu-berlin.de/resources/models"
 
@@ -149,3 +129,9 @@ class TextClassifier(flair.nn.DefaultClassifier[Sentence]):
     @property
     def label_type(self):
         return self._label_type
+
+    @classmethod
+    def load(cls, model_path: Union[str, Path, Dict[str, Any]]) -> "TextClassifier":
+        from typing import cast
+
+        return cast("TextClassifier", super().load(model_path=model_path))
